"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var operators_1 = require("rxjs/operators");
var core_1 = require("@angular/core");
var platform_browser_1 = require("@angular/platform-browser");
var object_hash_1 = require("object-hash");
var rxjs_1 = require("rxjs");
var firestore_1 = require("angularfire2/firestore");
var browser_firebase_fs_common_1 = require("./browser.firebase.fs.common");
var server_firebase_fs_common_1 = require("./server.firebase.fs.common");
// tslint:disable:no-this
// tslint:disable-next-line:no-class
var UniversalFirestoreService = /** @class */ (function () {
    function UniversalFirestoreService(ts, afs, appRef) {
        var _this = this;
        this.ts = ts;
        this.afs = afs;
        // tslint:disable-next-line:readonly-keyword
        this.readFromCache = true;
        appRef.isStable
            .pipe(operators_1.filter(Boolean), operators_1.take(1))
            .subscribe(function () { return _this.turnOffCache(); });
    }
    UniversalFirestoreService.prototype.turnOffCache = function () {
        // tslint:disable-next-line:no-object-mutation
        this.readFromCache = false;
    };
    UniversalFirestoreService.prototype.universalDoc = function (path) {
        var url = server_firebase_fs_common_1.constructFsUrl(browser_firebase_fs_common_1.extractFsHostFromLib(this.afs), path);
        var cached = this.ts.get(server_firebase_fs_common_1.makeFirestoreStateTransferKey(url), undefined);
        var base = this.afs.doc(path).valueChanges();
        return this.readFromCache && cached
            ? base.pipe(operators_1.startWith(cached), operators_1.distinctUntilChanged(function (x, y) { return object_hash_1.sha1(x) === object_hash_1.sha1(y); }), operators_1.catchError(function (err) { return rxjs_1.of(undefined); }))
            : base;
    };
    UniversalFirestoreService.prototype.universalCollection = function (path, queryFn) {
        var url = server_firebase_fs_common_1.constructFsUrl(browser_firebase_fs_common_1.extractFsHostFromLib(this.afs), path, true);
        var cached = this.ts.get(server_firebase_fs_common_1.makeFirestoreStateTransferKey(url), []);
        var base = this.afs.collection(path, queryFn).valueChanges();
        return this.readFromCache && cached.length > 0
            ? base.pipe(operators_1.startWith(cached), operators_1.distinctUntilChanged(function (x, y) { return object_hash_1.sha1(x) === object_hash_1.sha1(y); }), operators_1.catchError(function (err) { return rxjs_1.of(cached); }))
            : base;
    };
    UniversalFirestoreService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    UniversalFirestoreService.ctorParameters = function () { return [
        { type: platform_browser_1.TransferState },
        { type: firestore_1.AngularFirestore },
        { type: core_1.ApplicationRef }
    ]; };
    return UniversalFirestoreService;
}());
exports.UniversalFirestoreService = UniversalFirestoreService;
//# sourceMappingURL=browser.firebase.fs.service.js.map