"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var platform_browser_1 = require("@angular/platform-browser");
var core_1 = require("@angular/core");
var firestore_1 = require("angularfire2/firestore");
var operators_1 = require("rxjs/operators");
var http_1 = require("@angular/common/http");
var rxjs_1 = require("rxjs");
var browser_firebase_fs_common_1 = require("./browser.firebase.fs.common");
var server_1 = require("../common/server");
var browser_1 = require("../common/browser");
var server_firebase_fs_common_1 = require("./server.firebase.fs.common");
var object_hash_1 = require("object-hash");
function mapOrderBy(ordBy) {
    return {
        field: {
            fieldPath: ordBy.field.segments.pop()
        },
        direction: ordBy.dir.name
            ? ordBy.dir.name === 'desc'
                ? 'DESCENDING'
                : 'ASCENDING'
            : 'DIRECTION_UNSPECIFIED'
    };
}
function mapOrderByCollection(ordBy) {
    return ordBy.map(function (a) { return mapOrderBy(a); });
}
function coerceType(type, value) {
    switch (type) {
        case 'booleanValue':
            return value;
        case 'stringValue':
            return value;
        case 'integerValue':
            return +value;
        case 'arrayValue':
            return value.values.map(function (obj) {
                return Object.keys(obj).reduce(function (acc, curr, idx) {
                    return coerceType(curr, obj[curr]);
                }, {});
            });
        case 'mapValue':
            return reduceFields(value.fields);
        case 'nullValue':
            return undefined;
        default:
            return undefined;
    }
}
function reduceFields(fields) {
    return Object.keys(fields).reduce(function (acc, curr) {
        var _a;
        var converted = extractFieldType(fields[curr]);
        var innerKey = Object.keys(converted).pop();
        return __assign({}, acc, (_a = {}, _a[curr] = innerKey && converted[innerKey], _a));
    }, {});
}
function extractFieldType(obj) {
    return Object.keys(obj).reduce(function (acc, curr) {
        var _a;
        return __assign({}, acc, (_a = {}, _a[curr] = coerceType(curr, obj[curr]), _a));
    }, {});
}
function getOpName(str) {
    switch (str) {
        case '==':
            return 'EQUAL';
        case '>':
            return 'GREATER_THAN';
        case '<':
            return 'LESS_THAN';
        case '>=':
            return 'GREATER_THAN_OR_EQUAL';
        case '<=':
            return 'LESS_THAN_OR_EQUAL';
    }
}
function lowerStrFirst(str) {
    return str.charAt(0).toLowerCase() + str.slice(1);
}
function mapFilterToFieldFilter(filter) {
    var _a;
    return {
        fieldFilter: {
            field: { fieldPath: filter.field.segments.join('/') },
            op: getOpName(filter.op.name),
            value: (_a = {},
                _a[lowerStrFirst(filter.value.constructor.name)] = filter.value
                    .internalValue,
                _a)
        }
    };
}
function composeFilter(filters) {
    return {
        compositeFilter: {
            op: 'AND',
            filters: filters
        }
    };
}
// tslint:disable:no-this
// tslint:disable-next-line:no-class
var ServerUniversalFirestoreService = /** @class */ (function () {
    function ServerUniversalFirestoreService(http, ts, afs, authToken, lru) {
        this.http = http;
        this.ts = ts;
        this.afs = afs;
        this.authToken = authToken;
        this.lru = lru;
    }
    ServerUniversalFirestoreService.prototype.universalDoc = function (path) {
        var url = server_firebase_fs_common_1.constructFsUrl(browser_firebase_fs_common_1.extractFsHostFromLib(this.afs), path);
        var params = browser_1.getParams({ auth: this.authToken });
        var cacheKey = server_1.getFullUrl(url, params);
        var tsKey = server_firebase_fs_common_1.makeFirestoreStateTransferKey(url);
        var cachedValue = server_1.attemptToGetLruCachedValue(cacheKey, this.lru);
        return cachedValue
            ? rxjs_1.of(cachedValue).pipe(operators_1.tap(browser_1.cacheInStateTransfer(this.ts, tsKey)))
            : this.http.get(url).pipe(operators_1.take(1), operators_1.map(function (res) { return res.fields; }), operators_1.map(reduceFields), operators_1.tap(browser_1.removeHttpInterceptorCache(this.ts, cacheKey)), operators_1.tap(browser_1.cacheInStateTransfer(this.ts, tsKey)), operators_1.tap(server_1.attemptToCacheInLru(cacheKey, this.lru)));
    };
    ServerUniversalFirestoreService.prototype.universalCollection = function (path, queryFn) {
        var url = server_firebase_fs_common_1.constructFsUrl(browser_firebase_fs_common_1.extractFsHostFromLib(this.afs), undefined, true);
        var tsKey = server_firebase_fs_common_1.makeFirestoreStateTransferKey(url);
        var ref = this.afs.firestore.collection(path);
        var query = (queryFn && queryFn(ref)) || ref;
        var limit = query._query.limit;
        var filters = query._query.filters;
        var orderBy = query._query.explicitOrderBy;
        var cacheKey = object_hash_1.sha1(__assign({}, query._query, { path: path }));
        var cachedValue = server_1.attemptToGetLruCachedValue(cacheKey, this.lru);
        var fieldFilters = filters.map(mapFilterToFieldFilter);
        var where = composeFilter(fieldFilters);
        var structuredQuery = {
            limit: limit,
            from: [{ collectionId: path }],
            orderBy: mapOrderByCollection(orderBy),
            where: where
        };
        var fbToken = undefined; //TODO: this.auth.getCustomFirebaseToken()
        var baseObs = fbToken !== undefined
            ? this.http.post(url, { structuredQuery: structuredQuery }, { headers: { Authorization: "Bearer " + fbToken } })
            : this.http.post(url, { structuredQuery: structuredQuery });
        return cachedValue
            ? rxjs_1.of(cachedValue).pipe(operators_1.tap(browser_1.cacheInStateTransfer(this.ts, tsKey)))
            : baseObs.pipe(operators_1.take(1), operators_1.map(function (docs) {
                return docs.filter(function (a) { return a.document; }).map(function (doc) {
                    return reduceFields(doc.document.fields);
                });
            }), operators_1.tap(browser_1.cacheInStateTransfer(this.ts, tsKey)), operators_1.tap(server_1.attemptToCacheInLru(cacheKey, this.lru)));
    };
    ServerUniversalFirestoreService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    ServerUniversalFirestoreService.ctorParameters = function () { return [
        { type: http_1.HttpClient },
        { type: platform_browser_1.TransferState },
        { type: firestore_1.AngularFirestore },
        { type: String, decorators: [{ type: core_1.Optional }, { type: core_1.Inject, args: [server_1.FIREBASE_USER_AUTH_TOKEN,] }] },
        { type: undefined, decorators: [{ type: core_1.Optional }, { type: core_1.Inject, args: [server_1.LRU_CACHE,] }] }
    ]; };
    return ServerUniversalFirestoreService;
}());
exports.ServerUniversalFirestoreService = ServerUniversalFirestoreService;
//# sourceMappingURL=server.firebase.fs.service.js.map